@@ -4,6 +4,7 @@ import (
 	"context"
 	"sync"
 
+	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
 	"github.com/skeema/skeema/internal/applier"
 	"github.com/skeema/skeema/internal/fs"
@@ -30,7 +31,7 @@ func init() {
 
 	cmd := mybase.NewCommand("push", summary, desc, PushHandler)
 
-	cmd.AddOptions("DDL generation",
+	cmd.AddOptions("SQL generation",
 		mybase.BoolOption("exact-match", 0, false, "Follow *.sql table definitions exactly, even for differences with no functional impact"),
 		mybase.BoolOption("compare-metadata", 0, false, "For stored programs, detect changes to creation-time sql_mode or DB collation"),
 		mybase.BoolOption("alter-validate-virtual", 0, false, "Apply a WITH VALIDATION clause to ALTER TABLEs affecting virtual columns"),
@@ -72,6 +73,21 @@ func init() {
 
 // PushHandler is the handler method for `skeema push`
 func PushHandler(cfg *mybase.Config) error {
+	// Set up some config overrides relating to --brief output mode:
+	// * --brief only affects `skeema diff` (aka `skeema push --dry-run`)
+	// * --brief automatically uses --skip-verify --skip-lint --allow-unsafe
+	// * --brief omits INFO-level logging, unless --debug was used
+	if !cfg.GetBool("dry-run") {
+		cfg.SetRuntimeOverride("brief", "0")
+	} else if cfg.GetBool("brief") {
+		cfg.SetRuntimeOverride("verify", "0")
+		cfg.SetRuntimeOverride("lint", "0")
+		cfg.SetRuntimeOverride("allow-unsafe", "1")
+		if !cfg.GetBool("debug") {
+			log.SetLevel(log.WarnLevel)
+		}
+	}
+
 	dir, err := fs.ParseDir(".", cfg)
 	if err != nil {
 		return err
@@ -83,8 +99,7 @@ func PushHandler(cfg *mybase.Config) error {
 	} else if concurrency < 1 {
 		return NewExitValue(CodeBadConfig, "concurrent-instances cannot be less than 1")
 	}
-	briefMode := dir.Config.GetBool("dry-run") && dir.Config.GetBool("brief")
-	printer := applier.NewPrinter(briefMode)
+	printer := applier.NewPrinter(dir.Config)
 
 	g, ctx := errgroup.WithContext(context.Background())
 	g.SetLimit(concurrency)
@@ -95,6 +110,7 @@ func PushHandler(cfg *mybase.Config) error {
 	for n := range groups {
 		tg := groups[n] // avoid loop iteration variable in closure below
 		g.Go(func() error {
+			defer panicHandler()
 			for _, t := range tg {
 				select {
 				case <-ctx.Done():
