@@ -1,7 +1,6 @@
 package applier
 
 import (
-	"errors"
 	"fmt"
 	"os"
 	"strconv"
@@ -9,7 +8,6 @@ import (
 
 	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
-	"github.com/skeema/skeema/internal/fs"
 	"github.com/skeema/skeema/internal/tengo"
 	"github.com/skeema/skeema/internal/util"
 )
@@ -19,6 +17,7 @@ import (
 // run directly against a DB.
 type DDLStatement struct {
 	stmt     string
+	compound bool
 	shellOut *util.ShellOut
 
 	instance      *tengo.Instance
@@ -71,8 +70,7 @@ func NewDDLStatement(diff tengo.ObjectDiff, mods tengo.StatementModifiers, targe
 	if ddl.stmt, err = diff.Statement(mods); tengo.IsForbiddenDiff(err) {
 		terminalWidth, _ := util.TerminalWidth(int(os.Stderr.Fd()))
 		commentedOutStmt := "  # " + util.WrapStringWithPadding(ddl.stmt, terminalWidth-29, "  # ")
-		errorText := fmt.Sprintf("Preventing execution of unsafe or potentially destructive statement:\n%s\nUse --allow-unsafe or --safe-below-size to permit this operation. For more information, see Safety Options section of --help.", commentedOutStmt)
-		return nil, errors.New(errorText) // Intentionally avoiding fmt.Errorf here to avoid golint complaining about capitalization
+		return nil, fmt.Errorf("Preventing execution of unsafe or potentially destructive statement:\n%s\nUse --allow-unsafe or --safe-below-size to permit this operation. For more information, see Safety Options section of --help.", commentedOutStmt)
 	} else if err != nil {
 		// Leave the error untouched/unwrapped to allow caller to handle appropriately
 		return nil, err
@@ -81,6 +79,13 @@ func NewDDLStatement(diff tengo.ObjectDiff, mods tengo.StatementModifiers, targe
 		return nil, nil
 	}
 
+	// Determine if the statement is a compound statement, requiring special
+	// delimiter handling in output. Only stored program diffs (e.g. procs, funcs)
+	// implement this interface; others never generate compound statements.
+	if compounder, ok := diff.(tengo.Compounder); ok && compounder.IsCompoundStatement() {
+		ddl.compound = true
+	}
+
 	if wrapper == "" {
 		ddl.connectParams = getConnectParams(diff, target.Dir.Config)
 	} else {
@@ -119,9 +124,7 @@ func NewDDLStatement(diff tengo.ObjectDiff, mods tengo.StatementModifiers, targe
 		}
 
 		if ddl.shellOut, err = util.NewInterpolatedShellOut(wrapper, variables); err != nil {
-			// Intentionally avoiding fmt.Errorf here to avoid golint complaining about capitalization
-			errorText := fmt.Sprintf("A fatal error occurred with pre-processing a DDL statement: %s.", err)
-			return nil, errors.New(errorText)
+			return nil, fmt.Errorf("A fatal error occurred with pre-processing a DDL statement: %w.", err)
 		}
 	}
 
@@ -221,27 +224,10 @@ func getConnectParams(diff tengo.ObjectDiff, config *mybase.Config) string {
 	return ""
 }
 
-// IsShellOut returns true if the DDL is to be executed via shelling out to an
-// external binary, or false if the DDL represents SQL to be executed directly
-// via a standard database connection.
-func (ddl *DDLStatement) IsShellOut() bool {
-	return (ddl.shellOut != nil)
-}
-
-// String returns a string representation of ddl. If an external command is in
-// use, the returned string will be prefixed with "\!", the MySQL CLI command
-// shortcut for "system" shellout.
-func (ddl *DDLStatement) String() string {
-	if ddl.IsShellOut() {
-		return fmt.Sprintf("\\! %s\n", ddl.shellOut)
-	}
-	return fs.AddDelimiter(ddl.stmt)
-}
-
 // Execute runs the DDL statement, either by running a SQL query against a DB,
 // or shelling out to an external program, as appropriate.
 func (ddl *DDLStatement) Execute() error {
-	if ddl.IsShellOut() {
+	if ddl.shellOut != nil {
 		return ddl.shellOut.Run()
 	}
 	db, err := ddl.instance.CachedConnectionPool(ddl.schemaName, ddl.connectParams)
@@ -251,3 +237,29 @@ func (ddl *DDLStatement) Execute() error {
 	_, err = db.Exec(ddl.stmt)
 	return err
 }
+
+// Statement returns a string representation of ddl. If an external command is
+// in use, the returned string will be prefixed with "\!", the MySQL CLI command
+// shortcut for "system" shellout.
+func (ddl *DDLStatement) Statement() string {
+	if ddl.shellOut != nil {
+		return "\\! " + ddl.shellOut.String()
+	}
+	return ddl.stmt
+}
+
+// ClientState returns a representation of the client state which would be
+// used in execution of the statement.
+func (ddl *DDLStatement) ClientState() ClientState {
+	cs := ClientState{
+		InstanceName: ddl.instance.String(),
+		SchemaName:   ddl.schemaName,
+		Delimiter:    ";",
+	}
+	if ddl.shellOut != nil {
+		cs.Delimiter = ""
+	} else if ddl.compound {
+		cs.Delimiter = "//"
+	}
+	return cs
+}
