@@ -10,15 +10,34 @@ import (
 	"github.com/skeema/skeema/internal/workspace"
 )
 
-// VerifyDiff verifies the result of all AlterTable values found in
-// diff.TableDiffs, confirming that applying the corresponding ALTER would
-// bring a table from the version currently in the instance to the version
-// specified in the filesystem.
-func VerifyDiff(diff *tengo.SchemaDiff, t *Target) error {
-	if !t.Dir.Config.GetBool("verify") || len(diff.TableDiffs) == 0 || t.briefOutput() {
-		return nil
-	}
+// VerifierOptions specifies configuration for the diff verification operation.
+// All fields are mandatory, even though some may be redundant with
+// WorkspaceOptions in some situations.
+type VerifierOptions struct {
+	AllAlters           bool // if false, only verify unsupported alter diffs; if true, verify all alter diffs
+	Flavor              tengo.Flavor
+	DefaultCharacterSet string
+	DefaultCollation    string
+	WorkspaceOptions    workspace.Options
+}
 
+// VerifierOptionsForTarget returns VerifierOptions based on the target's
+// configuration.
+func VerifierOptionsForTarget(t *Target) (opts VerifierOptions, err error) {
+	opts = VerifierOptions{
+		AllAlters:           t.Dir.Config.GetBool("verify"),
+		Flavor:              t.Instance.Flavor(),
+		DefaultCharacterSet: t.Dir.Config.Get("default-character-set"),
+		DefaultCollation:    t.Dir.Config.Get("default-collation"),
+	}
+	opts.WorkspaceOptions, err = workspace.OptionsForDir(t.Dir, t.Instance)
+	return
+}
+
+// VerifyDiff verifies the result of AlterTable values found in diff.TableDiffs,
+// confirming that applying the corresponding ALTER would bring a table from the
+// version currently in the instance to the version specified in the filesystem.
+func VerifyDiff(diff *tengo.SchemaDiff, vopts VerifierOptions) error {
 	// If diff contains no ALTER TABLEs, nothing to verify
 	altersInDiff := diff.FilteredTableDiffs(tengo.DiffTypeAlter)
 	if len(altersInDiff) == 0 {
@@ -43,49 +62,58 @@ func VerifyDiff(diff *tengo.SchemaDiff, t *Target) error {
 		StrictForeignKeyNaming: true,                         // ditto
 		StrictColumnDefinition: true,                         // ditto (only affects MySQL 8 edge cases)
 		SkipPreDropAlters:      true,                         // ignore DROP PARTITIONs that were only generated to speed up a DROP TABLE
-		Flavor:                 t.Instance.Flavor(),
+		Flavor:                 vopts.Flavor,
 	}
 	if mods.Flavor.Matches(tengo.FlavorMySQL55) {
 		mods.AlgorithmClause = "" // MySQL 5.5 doesn't support ALGORITHM clause
 	}
 
 	// Gather CREATE and ALTER for modified tables, and put into a LogicalSchema,
-	// which we then materialize into a real schema using a workspace
-	logicalSchema := &fs.LogicalSchema{
-		CharSet:   t.Dir.Config.Get("default-character-set"),
-		Collation: t.Dir.Config.Get("default-collation"),
-		Creates:   make(map[tengo.ObjectKey]*fs.Statement),
-		Alters:    make([]*fs.Statement, 0),
-	}
+	// which we then materialize into a real schema using a workspace.
+	// Even if verify is disabled, we still must look for unsupported diffs, to
+	// potentially mark some as supported (if they generate non-blank SQL which
+	// properly verifies due to not actually touching unsupported features)
+	logicalSchema := fs.NewLogicalSchema()
+	logicalSchema.CharSet = vopts.DefaultCharacterSet
+	logicalSchema.Collation = vopts.DefaultCollation
 	desiredTables := make(map[string]*tengo.Table)
+	unsupportedTables := make(map[string]*tengo.TableDiff)
 	for _, td := range altersInDiff {
 		stmt, err := td.Statement(mods)
-		if stmt != "" && err == nil {
-			// Note: sometimes a table's diff gets split into multiple ALTERs, but this
-			// logic can ignore that fact. If there are redundant AddStatement calls for
-			// one CREATE, the first AddStatement for that CREATE succeeds and the
-			// subsequent duplicate CREATEs error, but that is harmless in this code path!
-			logicalSchema.AddStatement(&fs.Statement{
-				Type:       fs.StatementTypeCreate,
-				Text:       td.From.CreateStatement,
-				ObjectType: tengo.ObjectTypeTable,
-				ObjectName: td.From.Name,
-			})
-			logicalSchema.AddStatement(&fs.Statement{
-				Type:       fs.StatementTypeAlter,
-				Text:       stmt,
-				ObjectType: tengo.ObjectTypeTable,
-				ObjectName: td.From.Name,
-			})
-			desiredTables[td.From.Name] = td.To
+		if stmt == "" {
+			continue
+		} else if err != nil && tengo.IsUnsupportedDiff(err) {
+			unsupportedTables[td.From.Name] = td
+		} else if !vopts.AllAlters {
+			continue
 		}
+
+		// Note: sometimes a table's diff gets split into multiple ALTERs, but this
+		// logic can ignore that fact. If there are redundant AddStatement calls for
+		// one CREATE, the first AddStatement for that CREATE succeeds and the
+		// subsequent duplicate CREATEs error, but that is harmless in this code path!
+		logicalSchema.AddStatement(&tengo.Statement{
+			Type:       tengo.StatementTypeCreate,
+			Text:       td.From.CreateStatement,
+			ObjectType: tengo.ObjectTypeTable,
+			ObjectName: td.From.Name,
+		})
+		logicalSchema.AddStatement(&tengo.Statement{
+			Type:       tengo.StatementTypeAlter,
+			Text:       stmt,
+			ObjectType: tengo.ObjectTypeTable,
+			ObjectName: td.From.Name,
+		})
+		desiredTables[td.From.Name] = td.To
 	}
 
-	opts, err := workspace.OptionsForDir(t.Dir, t.Instance)
-	if err != nil {
-		return err
+	// Return early if --verify was disabled and there were no verifiable
+	// unsupported tables
+	if len(desiredTables) == 0 {
+		return nil
 	}
-	wsSchema, err := workspace.ExecLogicalSchema(logicalSchema, opts)
+
+	wsSchema, err := workspace.ExecLogicalSchema(logicalSchema, vopts.WorkspaceOptions)
 	if err == nil && len(wsSchema.Failures) > 0 {
 		err = wsSchema.Failures[0]
 	}
@@ -106,7 +134,12 @@ func VerifyDiff(diff *tengo.SchemaDiff, t *Target) error {
 	mods.AlgorithmClause = ""
 	actualTables := wsSchema.TablesByName()
 	for name, desiredTable := range desiredTables {
-		if err := verifyTable(actualTables[name], desiredTable, mods); err != nil {
+		// If an unsupported diff passes verification, mark it as supported, but
+		// otherwise we can just ignore any error from an unsupported diff.
+		td, wasUnsupported := unsupportedTables[name]
+		if err := verifyTable(actualTables[name], desiredTable, mods); err == nil && wasUnsupported {
+			td.MarkSupported()
+		} else if err != nil && !wasUnsupported {
 			return err
 		}
 	}
