@@ -2,7 +2,6 @@ package fs
 
 import (
 	"fmt"
-	"io/ioutil"
 	"net/url"
 	"os"
 	"path/filepath"
@@ -12,6 +11,7 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/VividCortex/mysqlerr"
 	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
 	"github.com/skeema/skeema/internal/tengo"
@@ -21,108 +21,17 @@ import (
 // Dir is a parsed representation of a directory that may have contained
 // a .skeema config file and/or *.sql files.
 type Dir struct {
-	Path               string
-	Config             *mybase.Config
-	OptionFile         *mybase.File
-	SQLFiles           []SQLFile
-	LogicalSchemas     []*LogicalSchema      // for now, always 0 or 1 elements; 2+ in same dir to be supported in future
-	ParseError         error                 // any fatal error found parsing dir's config or contents
-	UnparsedStatements []*Statement          // statements with unknown type / not supported by this package
-	IgnorePatterns     []tengo.ObjectPattern // regexes for matching objects that should be ignored
-	repoBase           string                // absolute path of containing repo, or topmost-found .skeema file
-}
-
-// LogicalSchema represents a set of statements from *.sql files in a directory
-// that all operated on the same schema. Note that Name is often blank, which
-// means "all SQL statements in this dir that don't have an explicit USE
-// statement before them". This "nameless" LogicalSchema is mapped to schema
-// names based on the "schema" option in the dir's OptionFile.
-type LogicalSchema struct {
-	Name      string
-	CharSet   string
-	Collation string
-	Creates   map[tengo.ObjectKey]*Statement
-	Alters    []*Statement // Alterations that are run after the Creates
-}
-
-// AddStatement adds the supplied statement into the appropriate data structure
-// within the receiver. This is useful when assembling a new logical schema.
-// An error will be returned if a duplicate CREATE object name/type pair is
-// added.
-func (logicalSchema *LogicalSchema) AddStatement(stmt *Statement) error {
-	switch stmt.Type {
-	case StatementTypeCreate:
-		if origStmt, already := logicalSchema.Creates[stmt.ObjectKey()]; already {
-			return DuplicateDefinitionError{
-				ObjectKey: stmt.ObjectKey(),
-				FirstFile: origStmt.File,
-				FirstLine: origStmt.LineNo,
-				DupeFile:  stmt.File,
-				DupeLine:  stmt.LineNo,
-			}
-		}
-		logicalSchema.Creates[stmt.ObjectKey()] = stmt
-		return nil
-	case StatementTypeAlter:
-		logicalSchema.Alters = append(logicalSchema.Alters, stmt)
-		return nil
-	default:
-		return nil
-	}
-}
-
-// LowerCaseNames adjusts logicalSchema in-place such that its object names are
-// forced to lower-case as appropriate for the supplied NameCaseMode.
-// An error will be returned if case-insensitivity would result in duplicate
-// objects with the same name and type.
-func (logicalSchema *LogicalSchema) LowerCaseNames(mode tengo.NameCaseMode) error {
-	switch mode {
-	case tengo.NameCaseLower: // lower_case_table_names=1
-		// Schema names and table names are forced lowercase in this mode
-		logicalSchema.Name = strings.ToLower(logicalSchema.Name)
-		newCreates := make(map[tengo.ObjectKey]*Statement, len(logicalSchema.Creates))
-		for k, stmt := range logicalSchema.Creates {
-			if k.Type == tengo.ObjectTypeTable {
-				k.Name = strings.ToLower(k.Name)
-				stmt.ObjectName = strings.ToLower(stmt.ObjectName)
-				if origStmt, already := newCreates[k]; already {
-					return DuplicateDefinitionError{
-						ObjectKey: stmt.ObjectKey(),
-						FirstFile: origStmt.File,
-						FirstLine: origStmt.LineNo,
-						DupeFile:  stmt.File,
-						DupeLine:  stmt.LineNo,
-					}
-				}
-			}
-			newCreates[k] = stmt
-		}
-		logicalSchema.Creates = newCreates
-
-	case tengo.NameCaseInsensitive: // lower_case_table_names=2
-		// Only view names are forced to lowercase in this mode, but Community Edition
-		// codebase does not support views, so nothing to lowercase here.
-		// However, with this mode we still need to ensure there aren't any duplicate
-		// table names in CREATEs after accounting for case-insensitive table naming.
-		lowerTables := make(map[string]*Statement)
-
-		for k, stmt := range logicalSchema.Creates {
-			if k.Type == tengo.ObjectTypeTable {
-				lowerName := strings.ToLower(k.Name)
-				if origStmt, already := lowerTables[lowerName]; already {
-					return DuplicateDefinitionError{
-						ObjectKey: stmt.ObjectKey(),
-						FirstFile: origStmt.File,
-						FirstLine: origStmt.LineNo,
-						DupeFile:  stmt.File,
-						DupeLine:  stmt.LineNo,
-					}
-				}
-				lowerTables[lowerName] = stmt
-			}
-		}
-	}
-	return nil
+	Path                  string
+	Config                *mybase.Config
+	OptionFile            *mybase.File
+	SQLFiles              map[string]*SQLFile   // .sql files, keyed by absolute file path, usually with file name lowercased
+	UnparsedStatements    []*tengo.Statement    // statements with unknown type / not supported by this package
+	NamedSchemaStatements []*tengo.Statement    // statements with explicit schema names: USE command or CREATEs with schema name qualifier
+	LogicalSchemas        []*LogicalSchema      // for now, always 0 or 1 elements; 2+ in same dir to be supported in future
+	IgnorePatterns        []tengo.ObjectPattern // regexes for matching objects that should be ignored
+	ParseError            error                 // any fatal error found parsing dir's config or contents
+	repoBase              string                // absolute path of containing repo, or topmost-found .skeema file
+	retainMapKeyCasing    bool                  // if true, map keys in SQLFiles retain original casing; used only when conflicting filenames found
 }
 
 // ParseDir parses the specified directory, including all *.sql files in it,
@@ -200,73 +109,42 @@ func (dir *Dir) HasFile(name string) (bool, error) {
 // nil, but some of the returned Dir values will have a non-nil ParseError if
 // any problems were encountered in that subdir.
 func (dir *Dir) Subdirs() ([]*Dir, error) {
-	fileInfos, err := ioutil.ReadDir(dir.Path)
+	entries, err := os.ReadDir(dir.Path)
 	if err != nil {
 		return nil, err
 	}
-	result := make([]*Dir, 0, len(fileInfos))
-	for _, fi := range fileInfos {
-		if fi.IsDir() && fi.Name()[0] != '.' {
-			sub := &Dir{
-				Path:     filepath.Join(dir.Path, fi.Name()),
-				Config:   dir.Config.Clone(),
-				repoBase: dir.repoBase,
+	result := make([]*Dir, 0, len(entries))
+	for _, entry := range entries {
+		if entry.IsDir() && entry.Name()[0] != '.' {
+			if sub, _ := dir.Subdir(entry.Name()); sub != nil {
+				result = append(result, sub)
 			}
-			sub.parseContents()
-			result = append(result, sub)
 		}
 	}
 	return result, nil
 }
 
-// CreateSubdir creates a subdirectory with the supplied name and optional
-// config file. If the directory already exists, it is an error if it already
-// contains any *.sql files or a .skeema file.
-func (dir *Dir) CreateSubdir(name string, optionFile *mybase.File) (*Dir, error) {
-	dirPath := filepath.Join(dir.Path, name)
-	if dir.OptionFile != nil && dir.OptionFile.SomeSectionHasOption("schema") {
-		return nil, ConfigErrorf("Cannot use dir %s: parent option file %s defines schema option", dirPath, dir.OptionFile)
-	} else if _, ok := dir.Config.Source("schema").(*mybase.File); ok {
-		return nil, ConfigErrorf("Cannot use dir %s: an ancestor option file defines schema option", dirPath)
-	}
-
-	if fi, err := os.Stat(dirPath); os.IsNotExist(err) {
-		err = os.MkdirAll(dirPath, 0777)
-		if err != nil {
-			return nil, fmt.Errorf("Unable to create directory %s: %s", dirPath, err)
-		}
-	} else if err != nil {
-		return nil, err
-	} else if !fi.IsDir() {
-		return nil, fmt.Errorf("Path %s already exists but is not a directory", dirPath)
-	} else {
-		// Existing dir: confirm it doesn't already have .skeema or *.sql files
-		fileInfos, err := ioutil.ReadDir(dirPath)
-		if err != nil {
-			return nil, err
-		}
-		for _, fi := range fileInfos {
-			if fi.Name() == ".skeema" {
-				return nil, fmt.Errorf("Cannot use dir %s: already has .skeema file", dirPath)
-			} else if strings.HasSuffix(fi.Name(), ".sql") {
-				return nil, fmt.Errorf("Cannot use dir %s: Already contains *.sql files", dirPath)
-			}
-		}
-	}
-
-	if optionFile != nil {
-		optionFile.Dir = dirPath
-		if err := optionFile.Write(false); err != nil {
-			return nil, fmt.Errorf("Cannot use dir %s: Unable to write to %s: %s", dirPath, optionFile.Path(), err)
-		}
-	}
-
+// Subdir returns a specific subdirectory of dir by name. If the named
+// subdirectory does not exist or is a non-directory, then a nil *Dir will be
+// returned alongside an error. In some other error conditions (such as a
+// problem parsing the directory's .skeema file or *.sql files), a non-nil *Dir
+// may be returned even alongside a non-nil error.
+func (dir *Dir) Subdir(name string) (*Dir, error) {
 	sub := &Dir{
-		Path:     dirPath,
+		Path:     filepath.Join(dir.Path, name),
 		Config:   dir.Config.Clone(),
 		repoBase: dir.repoBase,
 	}
 	sub.parseContents()
+	if sub.ParseError != nil {
+		// See if the parse error was caused by a more fundamental problem with the
+		// path; in these cases don't bother returning a non-nil Dir
+		if fi, err := os.Stat(sub.Path); err != nil {
+			return nil, err
+		} else if !fi.IsDir() {
+			return nil, fmt.Errorf("Path %s is not a directory", sub.Path)
+		}
+	}
 	return sub, sub.ParseError
 }
 
@@ -274,7 +152,7 @@ func (dir *Dir) CreateSubdir(name string, optionFile *mybase.File) (*Dir, error)
 // already has an option file.
 func (dir *Dir) CreateOptionFile(optionFile *mybase.File) (err error) {
 	if dir.OptionFile != nil {
-		return fmt.Errorf("Directory %s already has an option file", dir)
+		return fmt.Errorf("directory %s already has .skeema file", dir)
 	}
 	optionFile.Dir = dir.Path
 	if err := optionFile.Write(false); err != nil {
@@ -284,6 +162,18 @@ func (dir *Dir) CreateOptionFile(optionFile *mybase.File) (err error) {
 		return err
 	}
 	dir.Config.AddSource(dir.OptionFile)
+
+	// If the option file configures a schema name, add an empty logical schema
+	// to the dir. (Normally this is handled by parseContents() if the dir's config
+	// has a schema, but clearly the dir didn't have an option file yet when that
+	// was called.)
+	if len(dir.LogicalSchemas) == 0 && dir.HasSchema() {
+		ls := NewLogicalSchema()
+		ls.CharSet = dir.Config.Get("default-character-set")
+		ls.Collation = dir.Config.Get("default-collation")
+		dir.LogicalSchemas = []*LogicalSchema{ls}
+	}
+
 	return nil
 }
 
@@ -320,6 +210,58 @@ func (dir *Dir) Port() (int, bool) {
 	return intValue, dir.Config.Supplied("port")
 }
 
+// FileFor returns a SQLFile associated with the supplied keyer. If keyer is a
+// *tengo.Statement with non-empty File field, that path will be used as-is.
+// Otherwise, FileFor returns the default location for the supplied keyer based
+// on its type and name. In either case, if no known SQLFile exists at that
+// location yet, FileFor will instantiate a new SQLFile value for it, but no
+// underlying filesystem file is created/written by this method.
+func (dir *Dir) FileFor(keyer tengo.ObjectKeyer) *SQLFile {
+	var dirPath, base string
+	if stmt, ok := keyer.(*tengo.Statement); ok && stmt.File != "" {
+		dirPath, base = filepath.Split(stmt.File)
+	} else {
+		dirPath = dir.Path
+		base = FileNameForObject(keyer.ObjectKey().Name)
+	}
+
+	// Build the real file path, as well as a version of the path usable as a
+	// map key. For looking up whether we already have a file at that path, the
+	// file name will be lowercased as a map key, *unless* the dir already had at
+	// least one naming conflict when the dir was initially processed (e.g. both
+	// "Foo.sql" and "foo.sql" already existed). We want to avoid introducing that
+	// scenario since on case-insensitive systems these will refer to the same
+	// file.
+	filePathReal := filepath.Join(dirPath, base)
+	var filePathKey string
+	if dir.retainMapKeyCasing {
+		filePathKey = filePathReal
+	} else {
+		filePathKey = filepath.Join(dirPath, strings.ToLower(base))
+	}
+
+	// No file yet at that path: return a new SQLFile, but no need to mark it
+	// dirty / actually create the file or write anything. The caller can handle
+	// that if they're adding a statement to the file.
+	if dir.SQLFiles[filePathKey] == nil {
+		dir.SQLFiles[filePathKey] = &SQLFile{
+			FilePath:   filePathReal,
+			Statements: []*tengo.Statement{},
+		}
+	}
+	return dir.SQLFiles[filePathKey]
+}
+
+// DirtyFiles returns a slice of SQLFiles that have been marked as dirty.
+func (dir *Dir) DirtyFiles() (result []*SQLFile) {
+	for _, sf := range dir.SQLFiles {
+		if sf.Dirty {
+			result = append(result, sf)
+		}
+	}
+	return
+}
+
 // Instances returns 0 or more tengo.Instance pointers, based on the
 // directory's configuration. The Instances will NOT be checked for
 // connectivity. However, if the configuration is invalid (for example, illegal
@@ -424,6 +366,17 @@ func (dir *Dir) FirstInstance() (*tengo.Instance, error) {
 func (dir *Dir) ValidateInstance(instance *tengo.Instance) error {
 	ok, err := instance.Valid()
 	if !ok {
+		if instance.Password == "" && tengo.IsDatabaseError(err, mysqlerr.ER_ACCESS_DENIED_ERROR) {
+			err = fmt.Errorf("%w\nNo password was supplied for this login attempt, but the server likely requires a password. For information on how to use Skeema's password option, see https://www.skeema.io/docs/options/#password", err)
+		} else if dir.Config.Changed("connect-options") {
+			if tengo.IsDatabaseError(err, mysqlerr.ER_SPECIFIC_ACCESS_DENIED_ERROR) {
+				err = fmt.Errorf("%w\nCheck your Skeema configuration for connect-options. Typically this error means one of your session variables requires privileges that your user lacks.\nFor more information, see https://www.skeema.io/docs/options/#connect-options", err)
+			} else if tengo.IsDatabaseError(err, mysqlerr.ER_UNKNOWN_SYSTEM_VARIABLE, mysqlerr.ER_INCORRECT_GLOBAL_LOCAL_VAR) {
+				err = fmt.Errorf("%w\nCheck your Skeema configuration for connect-options. Typically this error means one of your session variable names has a typo, or the variable does not exist at the session scope in your specific server version.", err)
+			} else if tengo.IsDatabaseError(err, mysqlerr.ER_WRONG_VALUE_FOR_VAR, mysqlerr.ER_WRONG_TYPE_FOR_VAR) {
+				err = fmt.Errorf("%w\nCheck your Skeema configuration for connect-options. Typically this error means one of your session variable values has a typo, or the value is not supported in your specific server version.", err)
+			}
+		}
 		return err
 	}
 
@@ -567,42 +520,6 @@ func (dir *Dir) HasSchema() bool {
 	return false
 }
 
-var reUseStatement = regexp.MustCompile(`(?i)\bUSE\b`)
-
-// NamedSchemaStatements returns a slice of Statements in the dir that are
-// a USE command, or are a CREATE that specify a schema name. Such statements
-// are not yet fully supported by most Skeema packages.
-func (dir *Dir) NamedSchemaStatements() []*Statement {
-	result := make([]*Statement, 0)
-	checkFiles := make(map[*TokenizedSQLFile]bool, 0)
-	for _, ls := range dir.LogicalSchemas {
-		if ls.Name != "" {
-			for _, stmt := range ls.Creates {
-				if stmt.ObjectQualifier != "" {
-					result = append(result, stmt)
-				} else {
-					checkFiles[stmt.FromFile] = true
-				}
-			}
-			for _, stmt := range ls.Alters {
-				if stmt.ObjectQualifier != "" {
-					result = append(result, stmt)
-				} else {
-					checkFiles[stmt.FromFile] = true
-				}
-			}
-		}
-	}
-	for tokenizedFile := range checkFiles {
-		for _, stmt := range tokenizedFile.Statements {
-			if stmt.Type == StatementTypeCommand && reUseStatement.MatchString(stmt.Text) {
-				result = append(result, stmt)
-			}
-		}
-	}
-	return result
-}
-
 // InstanceDefaultParams returns a param string for use in constructing a
 // DSN. Any overrides specified in the config for this dir will be taken into
 // account. The returned string will already be in the correct format (HTTP
@@ -792,24 +709,45 @@ func (dir *Dir) parseContents() {
 		return
 	}
 
-	// Tokenize and parse any *.sql files
-	if dir.SQLFiles, dir.ParseError = sqlFiles(dir.Path, dir.repoBase); dir.ParseError != nil {
+	// See what *.sql files are here
+	var sqlFileNames []string
+	if sqlFileNames, dir.ParseError = sqlFiles(dir.Path, dir.repoBase); dir.ParseError != nil {
 		return
 	}
+
+	// See if there are any case-insensitive file name conflicts, since that
+	// affects how we key the file names. We seek to avoid introducing any new
+	// conflicts: if tables `Foo` and `foo` both exist, we normally want to put
+	// them in the same file to avoid problems on Mac/Windows, UNLESS the dir
+	// ALREADY has a situation where "Foo.sql" and "foo.sql" both exist.
+	dir.SQLFiles = make(map[string]*SQLFile, len(sqlFileNames))
+	for _, fileName := range sqlFileNames {
+		normalizedPath := filepath.Join(dir.Path, strings.ToLower(fileName))
+		if _, already := dir.SQLFiles[normalizedPath]; already {
+			dir.retainMapKeyCasing = true
+			dir.SQLFiles = make(map[string]*SQLFile, len(sqlFileNames))
+			break
+		}
+		dir.SQLFiles[normalizedPath] = nil
+	}
+
+	// Tokenize, parse, and track all *.sql files
 	logicalSchemasByName := make(map[string]*LogicalSchema)
-	for _, sf := range dir.SQLFiles {
-		tokenizedFile, err := sf.Tokenize()
-		if err != nil {
-			// Treat errors from Tokenize as fatal. This includes: i/o error opening or
-			// reading the .sql file; file had unterminated quote or backtick or comment.
+	for _, fileName := range sqlFileNames {
+		sf := &SQLFile{
+			FilePath: filepath.Join(dir.Path, fileName),
+		}
+		sf.Statements, dir.ParseError = tengo.ParseStatementsInFile(sf.FilePath)
+		if dir.ParseError != nil {
+			// Treat errors here as fatal. This includes: i/o error opening or reading
+			// the .sql file; file had unterminated quote or backtick or comment.
 			// These are all problematic, since if the caller otherwise just skipped the
 			// statements in the file, it could result in the caller emitting DROP
 			// statements incorrectly -- not good if the root cause is just an unclosed
 			// quote for example.
-			dir.ParseError = err
 			return
 		}
-		for _, stmt := range tokenizedFile.Statements {
+		for _, stmt := range sf.Statements {
 			// Statements that are ignored due to ignore-table, ignore-proc, etc are
 			// simply not placed into a LogicalSchema, so that all other logic won't
 			// interact with them
@@ -818,60 +756,61 @@ func (dir *Dir) parseContents() {
 			}
 
 			if _, ok := logicalSchemasByName[stmt.Schema()]; !ok {
-				logicalSchemasByName[stmt.Schema()] = &LogicalSchema{
-					Creates: make(map[tengo.ObjectKey]*Statement),
-				}
+				logicalSchemasByName[stmt.Schema()] = NewLogicalSchema()
 			}
 			dir.ParseError = logicalSchemasByName[stmt.Schema()].AddStatement(stmt)
 			if dir.ParseError != nil {
 				return
 			}
-			if stmt.Type == StatementTypeUnknown {
+			if stmt.Type == tengo.StatementTypeUnknown {
 				// Statements which could not be parsed, meaning of an unsupported statement
-				// type (e.g. INSERTs), are simply ignored. This is not fatal, since it is
+				// type (e.g. SELECTs), are simply ignored. This is not fatal, since it is
 				// quite rare for a typo to trigger this -- only happens when misspelling
 				// CREATE or the object type for example.
 				dir.UnparsedStatements = append(dir.UnparsedStatements, stmt)
-			} else if stmt.Type == StatementTypeLexError || stmt.Type == StatementTypeForbidden {
-				// Statements with lexer errors, meaning invalid characters, are treated as
-				// fatal. This can be indicative of a bug in the grammar, or of a normally-
-				// valid statement which has an illegal typo such as an invalid character
-				// mid-statement.
-				// Statements of unsupported form CREATE TABLE ... SELECT are also treated
-				// as fatal.
-				dir.ParseError = SQLContentsError(stmt.Error.Error())
-				return
+			} else if stmt.ObjectQualifier != "" || (stmt.Type == tengo.StatementTypeCommand && len(stmt.Text) > 4 && strings.EqualFold(stmt.Text[0:3], "use")) {
+				// Statements which refer to specific schema names can be problematic, since
+				// this conflicts with the ability to specify the schema name dynamically
+				// in the .skeema config file.
+				dir.NamedSchemaStatements = append(dir.NamedSchemaStatements, stmt)
 			}
 		}
-	}
-
-	// If there are no *.sql files, but .skeema defines a schema name, create an
-	// empty LogicalSchema. This permits `skeema pull` to work properly on a
-	// formerly-empty schema, for example.
-	if len(logicalSchemasByName) == 0 && dir.HasSchema() {
-		dir.LogicalSchemas = []*LogicalSchema{
-			{
-				Creates:   make(map[tengo.ObjectKey]*Statement),
-				CharSet:   dir.Config.Get("default-character-set"),
-				Collation: dir.Config.Get("default-collation"),
-			},
+		var filePathKey string
+		if dir.retainMapKeyCasing {
+			filePathKey = sf.FilePath
+		} else {
+			filePathKey = filepath.Join(dir.Path, strings.ToLower(fileName))
 		}
-		return
+		dir.SQLFiles[filePathKey] = sf
 	}
 
-	// Put any non-empty logical schemas into the dir, with the blank-named one
-	// always in the first position
+	// Prune any logical schema which didn't have any relevant statements (e.g.
+	// only had commands like USE, or statements that Skeema cannot parse)
+	for name, ls := range logicalSchemasByName {
+		if ls.Empty() {
+			delete(logicalSchemasByName, name)
+		}
+	}
+
+	// If there are no non-empty logical schemas in the directory, but .skeema
+	// defines a schema name, create an empty LogicalSchema. This permits
+	// `skeema pull` to work properly on a formerly-empty schema, for example.
+	if len(logicalSchemasByName) == 0 && dir.HasSchema() {
+		logicalSchemasByName[""] = NewLogicalSchema()
+	}
+
+	// Place logical schemas into the dir, with the blank-named one always in
+	// the first position if it exists
 	dir.LogicalSchemas = make([]*LogicalSchema, 0, len(logicalSchemasByName))
-	if ls, ok := logicalSchemasByName[""]; ok && len(ls.Creates) > 0 {
+	if ls, ok := logicalSchemasByName[""]; ok {
 		ls.CharSet = dir.Config.Get("default-character-set")
 		ls.Collation = dir.Config.Get("default-collation")
 		dir.LogicalSchemas = append(dir.LogicalSchemas, ls)
+		delete(logicalSchemasByName, "")
 	}
 	for name, ls := range logicalSchemasByName {
-		if name != "" && len(ls.Creates) > 0 {
-			ls.Name = name
-			dir.LogicalSchemas = append(dir.LogicalSchemas, ls)
-		}
+		ls.Name = name
+		dir.LogicalSchemas = append(dir.LogicalSchemas, ls)
 	}
 
 	// If the dir's configuration includes "password" with no =value, and the dir
@@ -921,16 +860,16 @@ func ParentOptionFiles(dirPath string, baseConfig *mybase.Config) ([]*mybase.Fil
 			// home directory, so stop early if we're already there
 			break
 		}
-		fileInfos, err := ioutil.ReadDir(curPath)
+		entries, err := os.ReadDir(curPath)
 		if err != nil {
 			// If we hit a dir we cannot read, halt early but don't consider this fatal
 			break
 		}
-		for _, fi := range fileInfos {
-			if fi.Name() == ".git" {
+		for _, entry := range entries {
+			if entry.Name() == ".git" {
 				repoBase = curPath
 				atRepoBase = true
-			} else if fi.Name() == ".skeema" && n > 0 {
+			} else if entry.Name() == ".skeema" && n > 0 {
 				// The second part of the above conditional ensures we ignore dirPath's own
 				// .skeema file, since that is handled separately in Dir.parseContents() in
 				// order to store it in dir.OptionFile
@@ -1032,20 +971,26 @@ func parseOptionFile(dirPath, repoBase string, baseConfig *mybase.Config) (*myba
 	return f, nil
 }
 
-// sqlFiles returns a slice of SQLFile for all *.sql files found in the supplied
-// path. This function does not recursively search subdirs, and does not parse
-// or validate the SQLFile contents in any way. An error will only be returned
-// if the directory cannot be read.
-// The repoBase affects evaluation of symlinks; any link destinations outside
-// of the repoBase are ignored.
-func sqlFiles(dirPath, repoBase string) ([]SQLFile, error) {
-	fileInfos, err := ioutil.ReadDir(dirPath)
+// sqlFiles returns a slice of relative file names for all *.sql files found in
+// the supplied directory path. This function does not recursively search
+// subdirs, and does not parse or validate the file contents in any way. An
+// error will only be returned if the directory cannot be read. The file names
+// are not manipulated here; the caller may need to adjust them before using as
+// map keys on a case-insensitive filesystem.
+// The repoBase affects evaluation of symlinks: any symlink destinations outside
+// of the repoBase are ignored and excluded from the result, and ditto for
+// symlinks that don't directly point to regular files (regardless of location).
+func sqlFiles(dirPath, repoBase string) (result []string, err error) {
+	entries, err := os.ReadDir(dirPath)
 	if err != nil {
 		return nil, err
 	}
-	result := make([]SQLFile, 0, len(fileInfos))
-	for _, fi := range fileInfos {
-		name := fi.Name()
+	for _, entry := range entries {
+		name := entry.Name()
+		fi, err := entry.Info()
+		if err != nil {
+			continue
+		}
 		// symlinks: verify it points to an existing file within repoBase. If it
 		// does not, or if any error occurs in any step in checking, skip it.
 		if fi.Mode()&os.ModeSymlink == os.ModeSymlink {
@@ -1062,17 +1007,18 @@ func sqlFiles(dirPath, repoBase string) ([]SQLFile, error) {
 			if !strings.HasPrefix(dest, repoBase) {
 				continue
 			}
-			if fi, err = os.Lstat(dest); err != nil { // using Lstat here to prevent symlinks-to-symlinks
+			// Make fi now point to the symlink destination. Lstat is used here to
+			// intentionally only allow one level of indirection; multi-hop symlinks
+			// will get filtered out by the fi.Mode().IsRegular() check below.
+			if fi, err = os.Lstat(dest); err != nil {
 				continue
 			}
 		}
-		destName := fi.Name()
-		if strings.HasSuffix(destName, ".sql") && fi.Mode().IsRegular() {
-			sf := SQLFile{
-				Dir:      dirPath,
-				FileName: name, // name relative to dirPath, NOT symlink destination!
-			}
-			result = append(result, sf)
+		if destName := fi.Name(); strings.HasSuffix(destName, ".sql") && fi.Mode().IsRegular() {
+			// Note we intentionally use name, not destName, here. For symlinks we want
+			// to return the symlink, not the destination, since the destination could
+			// be in a different directory.
+			result = append(result, name)
 		}
 	}
 	return result, nil
@@ -1104,22 +1050,3 @@ func (ce ConfigError) ExitCode() int {
 func ConfigErrorf(format string, a ...any) ConfigError {
 	return ConfigError{err: fmt.Errorf(format, a...)}
 }
-
-// DuplicateDefinitionError is an error returned when Dir.parseContents()
-// encounters multiple CREATE statements for the same exact object.
-type DuplicateDefinitionError struct {
-	ObjectKey tengo.ObjectKey
-	FirstFile string
-	FirstLine int
-	DupeFile  string
-	DupeLine  int
-}
-
-// Error satisfies the builtin error interface.
-func (dde DuplicateDefinitionError) Error() string {
-	return fmt.Sprintf("%s defined multiple times in same directory: %s line %d and %s line %d",
-		dde.ObjectKey,
-		dde.FirstFile, dde.FirstLine,
-		dde.DupeFile, dde.DupeLine,
-	)
-}
