@@ -89,7 +89,7 @@ func TestInstanceBuildParamString(t *testing.T) {
 		}
 
 		// can't compare strings directly since order may be different
-		result := instance.buildParamString(addOptions)
+		result := instance.BuildParamString(addOptions)
 		parsedResult, err := url.ParseQuery(result)
 		if err != nil {
 			t.Fatalf("url.ParseQuery(\"%s\") returned error: %s", result, err)
@@ -148,7 +148,7 @@ func TestInstanceIntrospectionParams(t *testing.T) {
 
 func (s TengoIntegrationSuite) TestInstanceConnect(t *testing.T) {
 	// Connecting to invalid schema should return an error
-	db, err := s.d.Connect("does-not-exist", "")
+	db, err := s.d.CachedConnectionPool("does-not-exist", "")
 	if err == nil {
 		t.Error("err is unexpectedly nil")
 	} else if db != nil {
@@ -156,7 +156,7 @@ func (s TengoIntegrationSuite) TestInstanceConnect(t *testing.T) {
 	}
 
 	// Connecting without specifying a default schema should be successful
-	db, err = s.d.Connect("", "")
+	db, err = s.d.CachedConnectionPool("", "")
 	if err != nil {
 		t.Errorf("Unexpected connection error: %s", err)
 	} else if db == nil {
@@ -164,7 +164,7 @@ func (s TengoIntegrationSuite) TestInstanceConnect(t *testing.T) {
 	}
 
 	// Connecting again with same schema and params should return the existing connection pool
-	db2, err := s.d.Connect("", "")
+	db2, err := s.d.CachedConnectionPool("", "")
 	if err != nil {
 		t.Errorf("Unexpected connection error: %s", err)
 	} else if db2 != db {
@@ -172,7 +172,7 @@ func (s TengoIntegrationSuite) TestInstanceConnect(t *testing.T) {
 	}
 
 	// Connecting again with different schema should return a different connection pool
-	db3, err := s.d.Connect("information_schema", "")
+	db3, err := s.d.CachedConnectionPool("information_schema", "")
 	if err != nil {
 		t.Errorf("Unexpected connection error: %s", err)
 	} else if db3 == db {
@@ -180,7 +180,7 @@ func (s TengoIntegrationSuite) TestInstanceConnect(t *testing.T) {
 	}
 
 	// Connecting again with different params should return a different connection pool
-	db4, err := s.d.Connect("information_schema", "foreign_key_checks=0&wait_timeout=20")
+	db4, err := s.d.CachedConnectionPool("information_schema", "foreign_key_checks=0&wait_timeout=20")
 	if err != nil {
 		t.Errorf("Unexpected connection error: %s", err)
 	} else if db4 == db || db4 == db3 {
@@ -195,6 +195,7 @@ func (s TengoIntegrationSuite) TestInstanceCanConnect(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Unexpected error from NewInstance: %s", err)
 	}
+	defer inst.CloseAll()
 
 	if ok, err := inst.CanConnect(); !ok || err != nil {
 		t.Fatalf("Unexpected return from CanConnect(): %t / %s", ok, err)
@@ -242,6 +243,7 @@ func (s TengoIntegrationSuite) TestInstanceValid(t *testing.T) {
 	if err != nil {
 		t.Fatalf("Unexpected error from NewInstance: %s", err)
 	}
+	defer inst.CloseAll()
 	if ok, err := inst.Valid(); !ok || err != nil {
 		t.Fatalf("Valid() unexpectedly returned %t / %v", ok, err)
 	}
@@ -285,7 +287,7 @@ func (s TengoIntegrationSuite) TestInstanceLockWaitTimeout(t *testing.T) {
 func (s TengoIntegrationSuite) TestInstanceCloseAll(t *testing.T) {
 	makePool := func(defaultSchema, params string) {
 		t.Helper()
-		db, err := s.d.Connect(defaultSchema, params)
+		db, err := s.d.CachedConnectionPool(defaultSchema, params)
 		if err != nil {
 			t.Fatalf("Unexpected connection error: %s", err)
 		} else if db == nil {
@@ -330,6 +332,8 @@ func (s TengoIntegrationSuite) TestInstanceFlavorVersion(t *testing.T) {
 		"mariadb:10.8":  FlavorMariaDB108,
 		"mariadb:10.9":  FlavorMariaDB109,
 		"mariadb:10.10": FlavorMariaDB1010,
+		"mariadb:10.11": FlavorMariaDB1011,
+		"mariadb:11.0":  FlavorMariaDB110,
 	}
 
 	// Determine expected Flavor value of the Dockerized instance being tested
@@ -377,9 +381,44 @@ func (s TengoIntegrationSuite) TestInstanceGrantChecks(t *testing.T) {
 	if !s.d.CanSkipBinlog() {
 		t.Fatal("Expected all Dockerized instances to be able to skip binlogs, but CanSkipBinlogs returned false")
 	}
-	if _, err := s.d.Connect("", "sql_log_bin=0"); err != nil {
+	if _, err := s.d.CachedConnectionPool("", "sql_log_bin=0"); err != nil {
 		t.Errorf("Error connecting with sql_log_bin=0: %v", err)
 	}
+
+	// Now create a less-privileged user, and confirm behavior. Note that for
+	// MariaDB 11.0, we intentionally grant SUPER since this no longer confers
+	// fine-grained privs.
+	var morePrivs string
+	if s.d.Flavor().Min(FlavorMariaDB110) {
+		morePrivs = ", SUPER"
+	}
+	db, err := s.d.CachedConnectionPool("", "")
+	if err != nil {
+		t.Fatalf("Unable to establish connection pool: %v", err)
+	}
+	dbExec := func(query string) {
+		t.Helper()
+		if _, err := db.Exec(query); err != nil {
+			t.Fatalf("Error running query on DockerizedInstance.\nQuery: %s\nError: %s", query, err)
+		}
+	}
+	t.Cleanup(func() {
+		dbExec("DROP USER 'notroot'@'%'")
+	})
+	dbExec(fmt.Sprintf("CREATE USER 'notroot'@'%%' IDENTIFIED BY '%s'", s.d.Password))
+	dbExec(fmt.Sprintf("GRANT SELECT, CREATE, DROP, ALTER, INDEX, CREATE ROUTINE, ALTER ROUTINE%s ON *.* TO 'notroot'@'%%'", morePrivs))
+	unprivDSN := strings.Replace(s.d.BaseDSN, s.d.User, "notroot", 1)
+	unprivInst, _ := NewInstance("mysql", unprivDSN)
+	if ok, err := unprivInst.Valid(); !ok {
+		t.Fatalf("Valid returned false, %v", err)
+	}
+	defer unprivInst.CloseAll()
+	if unprivInst.CanSkipBinlog() {
+		t.Error("CanSkipBinlog unexpectedly returned true for unprivileged user")
+	}
+	if _, err := unprivInst.ConnectionPool("", "sql_log_bin=0"); err == nil {
+		t.Error("Expected error connecting with sql_log_bin=0, but err was nil")
+	}
 }
 
 // TestInstanceGrantChecksRegexes provides unit testing coverage of the regexes
@@ -416,7 +455,7 @@ func TestInstanceGrantChecksRegexes(t *testing.T) {
 		"GRANT SELECT, INSERT, UPDATE, DELETE, CREATE, DROP, RELOAD, SHUTDOWN, PROCESS, FILE, REFERENCES, INDEX, ALTER, SHOW DATABASES, SUPER, CREATE TEMPORARY TABLES, LOCK TABLES, EXECUTE, REPLICATION SLAVE, REPLICATION CLIENT, CREATE VIEW, SHOW VIEW, CREATE ROUTINE, ALTER ROUTINE, CREATE USER, EVENT, TRIGGER, CREATE TABLESPACE, CREATE ROLE, DROP ROLE ON *.* TO `foo`@`%`",
 		"GRANT APPLICATION_PASSWORD_ADMIN,AUDIT_ADMIN,BACKUP_ADMIN,BINLOG_ADMIN,BINLOG_ENCRYPTION_ADMIN,CLONE_ADMIN,CONNECTION_ADMIN,ENCRYPTION_KEY_ADMIN,GROUP_REPLICATION_ADMIN,INNODB_REDO_LOG_ARCHIVE,PERSIST_RO_VARIABLES_ADMIN,REPLICATION_APPLIER,REPLICATION_SLAVE_ADMIN,RESOURCE_GROUP_ADMIN,RESOURCE_GROUP_USER,ROLE_ADMIN,SERVICE_CONNECTION_ADMIN,SESSION_VARIABLES_ADMIN,SET_USER_ID,SYSTEM_USER,SYSTEM_VARIABLES_ADMIN,TABLE_ENCRYPTION_ADMIN,XA_RECOVER_ADMIN ON *.* TO `foo`@`%`",
 		"GRANT BINLOG ADMIN ON *.* TO 'foo'@'%'", // MariaDB 10.5+, not to be confused with MySQL 8.0's BINLOG_ADMIN with an underscore!
-		"GRANT SUPER ON *.* TO 'foo'@'%'",
+		"GRANT SUPER ON *.* TO 'foo'@'%'",        // inst is FlavorUnknown, so it won't trip on MariaDB 11's SUPER change
 	}
 	for n, grant := range binlogSkipGrants {
 		inst.grants = []string{}
@@ -657,7 +696,7 @@ func (s TengoIntegrationSuite) TestInstanceDropTablesDeadlock(t *testing.T) {
 
 	s.SourceTestSQL(t, "integration-ext.sql", "rows.sql")
 
-	db, err := s.d.Connect("", "foreign_key_checks=0")
+	db, err := s.d.CachedConnectionPool("", "foreign_key_checks=0")
 	if err != nil {
 		t.Fatalf("Unable to connect to DockerizedInstance: %s", err)
 	}
@@ -705,7 +744,7 @@ func (s TengoIntegrationSuite) TestInstanceDropTablesSkipsViews(t *testing.T) {
 		FROM     information_schema.partitions p
 		WHERE    p.table_schema = 'testing' AND
 		         p.table_name LIKE 'view%'`
-	db, err := s.d.Connect("", "")
+	db, err := s.d.CachedConnectionPool("", "")
 	if err != nil {
 		t.Fatalf("Unexpected error obtaining connection pool: %v", err)
 	}
