@@ -9,10 +9,12 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"net/url"
 	"sync"
 	"time"
 
 	"github.com/VividCortex/mysqlerr"
+	"github.com/go-sql-driver/mysql"
 	"github.com/jmoiron/sqlx"
 	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
@@ -125,7 +127,26 @@ func OptionsForDir(dir *fs.Dir, instance *tengo.Instance) (Options, error) {
 		opts.Type = TypeLocalDocker
 		opts.Flavor = tengo.ParseFlavor(dir.Config.Get("flavor"))
 		opts.SkipBinlog = true
-		if instance != nil {
+		if instance == nil {
+			// Without an instance, we just take the directory's default params config
+			// and apply tls=false on top, since we know the Dockerized instance will be
+			// on the local machine.
+			defaultParams, err := dir.InstanceDefaultParams()
+			if err != nil {
+				return Options{}, err
+			}
+			opts.DefaultConnParams = tengo.MergeParamStrings(defaultParams, "tls=false")
+		} else {
+			// With an instance, we can copy the instance's default params (which
+			// typically came from connect-options / dir.InstanceDefaultParams anyway),
+			// sql_mode, lower_case_table_names, and (if needed) flavor.
+			// Note that we're manually shoving the instance's sql_mode into the params;
+			// we need it present regardless of whether connect-options set it explicitly.
+			// Many companies use non-default global sql_mode, especially on RDS, and we
+			// want the Dockerized instance to match. We're also disabling tls since
+			// Dockerized instance is local and instance probably has tls=preferred.
+			overrides := "tls=false&sql_mode=" + url.QueryEscape("'"+instance.SQLMode()+"'")
+			opts.DefaultConnParams = instance.BuildParamString(overrides)
 			opts.NameCaseMode = instance.NameCaseMode()
 			if !opts.Flavor.Known() {
 				opts.Flavor = instance.Flavor().Family()
@@ -139,9 +160,6 @@ func OptionsForDir(dir *fs.Dir, instance *tengo.Instance) (Options, error) {
 		} else if cleanup == "destroy" {
 			opts.CleanupAction = CleanupActionDestroy
 		}
-		if opts.DefaultConnParams, err = dir.InstanceDefaultParams(); err != nil {
-			return Options{}, err
-		}
 	} else {
 		opts.Type = TypeTempSchema
 		opts.Instance = instance
@@ -221,9 +239,9 @@ func RegisterShutdownFunc(f ShutdownFunc) {
 }
 
 // StatementError represents a problem that occurred when executing a specific
-// fs.Statement in a Workspace.
+// tengo.Statement in a Workspace.
 type StatementError struct {
-	*fs.Statement
+	*tengo.Statement
 	Err error
 }
 
@@ -240,6 +258,16 @@ func (se *StatementError) String() string {
 	return se.Error()
 }
 
+// ErrorNumber returns the server error code corresponding to Err if it is a
+// driver-provided error. Otherwise, it returns 0.
+func (se *StatementError) ErrorNumber() uint16 {
+	var merr *mysql.MySQLError
+	if errors.As(se.Err, &merr) {
+		return merr.Number
+	}
+	return 0
+}
+
 // Schema captures the result of executing the SQL from an fs.LogicalSchema
 // in a workspace, and then introspecting the resulting schema. It wraps the
 // introspected tengo.Schema alongside the original fs.LogicalSchema and any
@@ -313,7 +341,7 @@ func ExecLogicalSchema(logicalSchema *fs.LogicalSchema, opts Options) (_ *Schema
 	}
 
 	// Run CREATEs in parallel, bounded by opts.Concurrency
-	creates := make(chan *fs.Statement, opts.Concurrency)
+	creates := make(chan *tengo.Statement, opts.Concurrency)
 	errs := make(chan error, opts.Concurrency)
 	go func() {
 		for _, stmt := range logicalSchema.Creates {
@@ -338,7 +366,7 @@ func ExecLogicalSchema(logicalSchema *fs.LogicalSchema, opts Options) (_ *Schema
 	// MySQL 8+ if FKs are present. Ditto with metadata lock wait timeouts.
 	// Also retry errors from CREATE TABLE...LIKE being run out-of-order (only once
 	// though; nested chains of CREATE TABLE...LIKE are unsupported)
-	sequentialStatements := []*fs.Statement{}
+	sequentialStatements := []*tengo.Statement{}
 	for n := 0; n < len(logicalSchema.Creates); n++ {
 		if err := <-errs; err != nil {
 			stmterr := err.(*StatementError)
@@ -365,7 +393,7 @@ func ExecLogicalSchema(logicalSchema *fs.LogicalSchema, opts Options) (_ *Schema
 	return wsSchema, err
 }
 
-func wrapFailure(statement *fs.Statement, err error) *StatementError {
+func wrapFailure(statement *tengo.Statement, err error) *StatementError {
 	stmtErr := &StatementError{
 		Statement: statement,
 	}
@@ -402,20 +430,20 @@ func getLock(instance *tengo.Instance, lockName string, maxWait time.Duration) (
 			case <-done:
 				err := lockConn.QueryRowContext(context.Background(), "SELECT RELEASE_LOCK(?)", lockName).Scan(&result)
 				if err != nil || result != 1 {
-					log.Warnf("%s: Failed to release lock, or lock released early due to connection being dropped: %s [%d]", instance, err, result)
+					log.Warnf("%s: Failed to release workspace lock, or lock released early due to connection being dropped: %s [%d]", instance, err, result)
 				}
 				return
 			case <-time.After(750 * time.Millisecond):
 				err := lockConn.QueryRowContext(context.Background(), "SELECT 1").Scan(&result)
 				if err != nil {
-					log.Warnf("%s: Lock released early due to connection being dropped: %s", instance, err)
+					log.Warnf("%s: Workspace lock released early due to connection being dropped: %s", instance, err)
 					return
 				}
 			}
 		}
 	}
 
-	var getLockResult int
+	var getLockResult, attempts int
 	start := time.Now()
 	for time.Since(start) < maxWait {
 		// Only using a timeout of 1 sec on each query to avoid potential issues with
@@ -427,6 +455,9 @@ func getLock(instance *tengo.Instance, lockName string, maxWait time.Duration) (
 			go connMaintainer()
 			return release, nil
 		}
+		if attempts++; attempts == 3 {
+			log.Warnf("Obtaining a workspace lock on %s is taking longer than expected. Some other Skeema process or thread may be holding the lock already. This operation will be re-attempted for up to %s total.", instance, maxWait)
+		}
 	}
-	return nil, errors.New("Unable to acquire lock")
+	return nil, errors.New("Unable to acquire lock before timeout")
 }
