@@ -7,6 +7,7 @@ import (
 	"strings"
 	"testing"
 
+	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
 	"github.com/skeema/skeema/internal/fs"
 	"github.com/skeema/skeema/internal/tengo"
@@ -88,21 +89,26 @@ func (s SkeemaIntegrationSuite) TestInitHandler(t *testing.T) {
 	mybase.AssertFileSetsOptions(t, dir.OptionFile, "host", "port", "connect-options")
 	mybase.AssertFileMissingOptions(t, dir.OptionFile, "schema", "default-character-set", "default-collation")
 
-	// init should fail if a parent dir has an invalid .skeema file
+	// init should fail if a parent dir (or working directory) has an invalid .skeema file
 	fs.MakeTestDirectory(t, "hasbadoptions")
 	fs.WriteTestFile(t, "hasbadoptions/.skeema", "invalid file will not parse")
 	s.handleCommand(t, CodeBadConfig, "hasbadoptions", "skeema init -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
 
-	// init should fail if the --dir specifies an existing non-directory file; or
-	// if the --dir already contains a subdir matching a schema name; or if the
-	// --dir already contains a .sql file and --schema was used to only do 1 level
+	// init should fail in any of various permutations where a non-directory file
+	// is present where a dir needs to be created; or if an existing dir contains
+	// a .skeema file and/or one or more *.sql files
 	fs.WriteTestFile(t, "nondir", "foo bar")
-	s.handleCommand(t, CodeBadConfig, ".", "skeema init --dir nondir -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
+	s.handleCommand(t, CodeCantCreate, ".", "skeema init --dir nondir -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
+	fs.WriteTestFile(t, "okdir/product", "foo bar")
+	s.handleCommand(t, CodeCantCreate, ".", "skeema init --dir okdir -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
 	fs.WriteTestFile(t, "alreadyexists/product/.skeema", "schema=product\n")
 	s.handleCommand(t, CodeCantCreate, ".", "skeema init --dir alreadyexists -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
-	fs.MakeTestDirectory(t, "hassql")
+	fs.WriteTestFile(t, "alreadyexists2/product/foo.sql", "CREATE TABLE foo (id int);\n")
+	s.handleCommand(t, CodeCantCreate, ".", "skeema init --dir alreadyexists2 -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
 	fs.WriteTestFile(t, "hassql/foo.sql", "foo")
 	s.handleCommand(t, CodeBadConfig, ".", "skeema init --dir hassql --schema product -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
+	fs.WriteTestFile(t, "hasoptionfile/.skeema", "# comment")
+	s.handleCommand(t, CodeBadConfig, ".", "skeema init --dir hasoptionfile --schema product -h %s -P %d", s.d.Instance.Host, s.d.Instance.Port)
 }
 
 func (s SkeemaIntegrationSuite) TestAddEnvHandler(t *testing.T) {
@@ -301,22 +307,24 @@ func (s SkeemaIntegrationSuite) TestLintHandler(t *testing.T) {
 	if len(productDir.SQLFiles) < 4 {
 		t.Fatalf("Unable to obtain *.sql files from %s", productDir)
 	}
+	var commentsFilePath string
 	rewriteFiles := func(includeSyntaxError bool) {
-		for n, sf := range productDir.SQLFiles {
-			contents := fs.ReadTestFile(t, sf.Path())
-			switch n {
-			case 0:
+		for _, sf := range productDir.SQLFiles {
+			contents := fs.ReadTestFile(t, sf.FilePath)
+			switch sf.FileName() {
+			case "comments.sql":
+				commentsFilePath = sf.FilePath
 				if includeSyntaxError {
 					contents = strings.Replace(contents, "DEFAULT", "DEFALUT", 1)
 				}
-			case 1:
+			case "posts.sql":
 				contents = strings.ToLower(contents)
-			case 2:
+			case "subscriptions.sql":
 				contents = strings.Replace(contents, "`", "", -1)
-			case 3:
+			case "users.sql":
 				contents = strings.Replace(contents, " ", "  ", -1)
 			}
-			fs.WriteTestFile(t, sf.Path(), contents)
+			fs.WriteTestFile(t, sf.FilePath, contents)
 		}
 	}
 	rewriteFiles(false)
@@ -333,14 +341,14 @@ func (s SkeemaIntegrationSuite) TestLintHandler(t *testing.T) {
 	// Manually restore the file with invalid SQL; the files should now verify,
 	// confirming that the fatal error did not prevent the other files from being
 	// reformatted; re-linting should yield no changes.
-	contents := fs.ReadTestFile(t, productDir.SQLFiles[0].Path())
-	fs.WriteTestFile(t, productDir.SQLFiles[0].Path(), strings.Replace(contents, "DEFALUT", "DEFAULT", 1))
+	contents := fs.ReadTestFile(t, commentsFilePath)
+	fs.WriteTestFile(t, commentsFilePath, strings.Replace(contents, "DEFALUT", "DEFAULT", 1))
 	s.verifyFiles(t, cfg, "../golden/init")
 	s.handleCommand(t, CodeSuccess, ".", "skeema lint")
 
 	// Files with SQL statements unsupported by this package should yield a
 	// warning, resulting in CodeDifferencesFound
-	fs.WriteTestFile(t, productDir.SQLFiles[0].Path(), "INSERT INTO foo (col1, col2) VALUES (123, 456)")
+	fs.WriteTestFile(t, commentsFilePath, "CALL some_proc(123, 234)")
 	s.handleCommand(t, CodeDifferencesFound, ".", "skeema lint")
 
 	// Directories that have invalid options should yield CodeBadConfig
@@ -370,22 +378,24 @@ func (s SkeemaIntegrationSuite) TestFormatHandler(t *testing.T) {
 	if len(productDir.SQLFiles) < 4 {
 		t.Fatalf("Unable to obtain *.sql files from %s", productDir)
 	}
+	var commentsFilePath string
 	rewriteFiles := func(includeSyntaxError bool) {
-		for n, sf := range productDir.SQLFiles {
-			contents := fs.ReadTestFile(t, sf.Path())
-			switch n {
-			case 0:
+		for _, sf := range productDir.SQLFiles {
+			contents := fs.ReadTestFile(t, sf.FilePath)
+			switch sf.FileName() {
+			case "comments.sql":
+				commentsFilePath = sf.FilePath
 				if includeSyntaxError {
 					contents = strings.Replace(contents, "DEFAULT", "DEFALUT", 1)
 				}
-			case 1:
+			case "posts.sql":
 				contents = strings.ToLower(contents)
-			case 2:
+			case "subscriptions.sql":
 				contents = strings.Replace(contents, "`", "", -1)
-			case 3:
+			case "users.sql":
 				contents = strings.Replace(contents, " ", "  ", -1)
 			}
-			fs.WriteTestFile(t, sf.Path(), contents)
+			fs.WriteTestFile(t, sf.FilePath, contents)
 		}
 	}
 	rewriteFiles(false)
@@ -404,14 +414,14 @@ func (s SkeemaIntegrationSuite) TestFormatHandler(t *testing.T) {
 	// Manually restore the file with invalid SQL; the files should now verify,
 	// confirming that the fatal error did not prevent the other files from being
 	// reformatted; re-formatting again should yield no changes.
-	contents := fs.ReadTestFile(t, productDir.SQLFiles[0].Path())
-	fs.WriteTestFile(t, productDir.SQLFiles[0].Path(), strings.Replace(contents, "DEFALUT", "DEFAULT", 1))
+	contents := fs.ReadTestFile(t, commentsFilePath)
+	fs.WriteTestFile(t, commentsFilePath, strings.Replace(contents, "DEFALUT", "DEFAULT", 1))
 	s.verifyFiles(t, cfg, "../golden/init")
 	s.handleCommand(t, CodeSuccess, ".", "skeema format")
 
 	// Files with SQL statements unsupported by this package should not affect
 	// exit code
-	fs.WriteTestFile(t, productDir.SQLFiles[0].Path(), "INSERT INTO foo (col1, col2) VALUES (123, 456)")
+	fs.WriteTestFile(t, commentsFilePath, "CALL some_proc(123, 234)")
 	s.handleCommand(t, CodeSuccess, ".", "skeema format --debug")
 
 	// Directories that have invalid options should yield CodeBadConfig
@@ -438,6 +448,10 @@ func (s SkeemaIntegrationSuite) TestDiffHandler(t *testing.T) {
 	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff")
 
 	// Confirm --brief works as expected
+	defer func() {
+		// --brief manipulates the log level, so we must restore it after
+		log.SetLevel(log.DebugLevel)
+	}()
 	oldStdout := os.Stdout
 	if outFile, err := os.Create("diff-brief.out"); err != nil {
 		t.Fatalf("Unable to redirect stdout to a file: %s", err)
@@ -792,23 +806,50 @@ func (s SkeemaIntegrationSuite) TestUnsupportedAlter(t *testing.T) {
 	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --debug")
 	s.handleCommand(t, CodePartialError, ".", "skeema push")
 
-	// diff/push still ok if *creating* or *dropping* unsupported table
+	// diff/push still ok if *creating* unsupported table
 	s.dbExec(t, "product", "DROP TABLE subscriptions")
 	s.assertTableMissing(t, "product", "subscriptions", "")
 	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff")
 	s.handleCommand(t, CodeSuccess, ".", "skeema push")
 	s.assertTableExists(t, "product", "subscriptions", "")
-	if err := os.Remove("mydb/product/subscriptions.sql"); err != nil {
+
+	// diff/push still ok if altering unsupported table to remove its unsupported
+	// feature, since the generated alter is verified, even with --skip-verify
+	contents = fs.ReadTestFile(t, "mydb/product/subscriptions.sql")
+	contents = strings.Replace(contents, "SUBPARTITION BY HASH (post_id)", "", 1)
+	contents = strings.Replace(contents, "SUBPARTITION BY HASH (`post_id`)", "", 1)
+	contents = strings.Replace(contents, "SUBPARTITIONS 2", "", 1)
+	if strings.Contains(contents, "SUBPARTITION") {
+		t.Fatalf("Failed to properly remove unsupported clause from subscriptions.sql -- contents:\n%s", contents)
+	} else {
+		fs.WriteTestFile(t, "mydb/product/subscriptions.sql", contents)
+	}
+	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --partitioning=modify --skip-verify")
+	s.handleCommand(t, CodeSuccess, ".", "skeema push --partitioning=modify --skip-verify")
+	s.handleCommand(t, CodeSuccess, ".", "skeema diff")
+
+	// Coverage for extra non-InnoDB warning text -- just ensuring no panic, and
+	// no need for allow-unsafe (since diff is not supported, due to USING BTREE
+	// clause on an index for a MyISAM table)
+	s.dbExec(t, "product", "ALTER TABLE users ENGINE=MyISAM")
+	contents = fs.ReadTestFile(t, "mydb/product/users.sql")
+	contents = strings.ReplaceAll(contents, "credits", "funds")
+	contents = strings.ReplaceAll(contents, "UNIQUE KEY `name` (`name`)", "UNIQUE KEY `name2` (`name`) USING BTREE")
+	contents = strings.ReplaceAll(contents, "InnoDB", "MyISAM")
+	fs.WriteTestFile(t, "mydb/product/users.sql", contents)
+	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff")
+
+	// Make the USING BTREE alter directly so that the table is unsupported, and
+	// then delete the users.sql file on the fs side. Confirm diff/push still ok
+	// for *dropping* unsupported table.
+	s.dbExec(t, "product", "ALTER TABLE users DROP KEY name, ADD UNIQUE KEY `name2` (`name`) USING BTREE")
+	if err := os.Remove("mydb/product/users.sql"); err != nil {
 		t.Fatalf("Unexpected error removing a file: %s", err)
 	}
 	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff --allow-unsafe")
 	s.handleCommand(t, CodeSuccess, ".", "skeema push --allow-unsafe")
-	s.assertTableMissing(t, "product", "subscriptions", "")
-
-	// coverage for non-InnoDB extra warning text -- just ensuring no panic and
-	// no unsafe-for-diff problem
-	s.sourceSQL(t, "unsupported2.sql")
-	s.handleCommand(t, CodeDifferencesFound, ".", "skeema diff")
+	s.assertTableMissing(t, "product", "users", "")
+	s.handleCommand(t, CodeSuccess, ".", "skeema diff")
 }
 
 func (s SkeemaIntegrationSuite) TestIgnoreOptions(t *testing.T) {
@@ -1306,15 +1347,8 @@ END`
 	funcs := schema.FunctionsByName()
 	if r2, ok := funcs["routine2"]; !ok {
 		t.Fatal("Unable to locate routine2")
-	} else {
-		var serverSQLMode string
-		db, _ := s.d.CachedConnectionPool("", "")
-		if err := db.QueryRow("SELECT @@global.sql_mode").Scan(&serverSQLMode); err != nil {
-			t.Fatalf("Unexpected error querying sql_mode: %s", err)
-		}
-		if r2.SQLMode != serverSQLMode {
-			t.Errorf("Expected routine2 to have sql_mode %s, instead found %s", serverSQLMode, r2.SQLMode)
-		}
+	} else if serverSQLMode := s.d.SQLMode(); r2.SQLMode != serverSQLMode {
+		t.Errorf("Expected routine2 to have sql_mode %s, instead found %s", serverSQLMode, r2.SQLMode)
 	}
 
 	// Lint that new file; confirm new formatting matches expectation.
